--------------------------------------------------------------------------------------
-- Engine Script for controlling additional engines
-- includes legacy DRB support
-- SRH 05/2011
-- may be reused with acknowledgement
--------------------------------------------------------------------------------------

-- lights table
-- Child name with trailing colon ( for main body objects leave blank, ie "" )
--		, node name, state at Headlights control value 0, 1, 2

gaLights = {
	{ "MainHeadlight1:", "lit", 0, 1, 0 },
	{ "MainHeadlight2:", "lit", 0, 0, 1 },
}
	
-- List of main exhaust emitters
-- leaving open to the possibility of emitters having different colours later
-- so liable to change format.
-- Note this doesn't deal with "special effect" emitters such as surge clouds
-- this is a list of the name of the CHILDREN, not the filenames
gaEmitters = {
	"Exhaust1",
	"Exhaust2",
	}
	
RPM_MIN = 450 -- idle rpm
RPM_MAX = 850 -- maximum RPM

-- Smoke rate variables
SM_MIN = 0.1 	-- minimum smoke rate, note that this variable is reversed, ie the bigger it is
				-- the more REDUCTION in smoke rate
SM_MAX = 0.012 	-- maximum smoke rate
SM_RANGE = SM_MIN - SM_MAX


-- Smoke darkness variables
SC_MIN = 0 		-- minimum smoke colour, 0 is black, 1 is white. This is the darkness at idle
SC_MAX = 0.8	-- maximum smoke colour, this is the darkness at full throttle

SC_RANGE = SC_MAX - SC_MIN

-- Smoke colour variables
SMOKE_PCT_R = 0.9	-- percentage of red in smoke
SMOKE_PCT_G = 0.9	-- percentage of green in smoke
SMOKE_PCT_B = 1		-- percentage of blue in smoke


-- Constant and variables block
-- backwards compatibility messages
ID_RPM = 1709
ID_RPMD = 1710


ID_START = 1711

ID_RPMPCT = 1720
ID_THROTTLEPCT = 1721
ID_PCTCTRL = 1725

gbUsePctRPM = false
gnThrottle = 0
gnRPMDrift = 0
gnPLights = 0
gbDirChange = false

_gaLights = {}


function rpm_as_pct( p_nRpm )
		return ( p_nRpm - RPM_MIN ) / RPM_RANGE
end

function pct_as_rpm ( p_nPct )
		return math.floor(( RPM_RANGE * p_nPct ) + RPM_MIN )
end

function checkLights( p_gValue )
	for k, v in pairs( _gaLights ) do
		Call( v[1].."ActivateNode", v[2], v[p_gValue+3] )
	end
end


function Setup()

end

function Initialise ()

	RPM_RANGE = RPM_MAX - RPM_MIN



	gPrevRPM = 0
	gPrevRPMDelta = 0
	gnRPMDrift = math.random( -15, 15 )
	
	if Call( "GetIsEngineWithKey" ) == 1 then
		Call( "SendConsistMessage", ID_PCTCTRL, 1, 0 )
		Call( "SendConsistMessage", ID_PCTCTRL, 1, 1 )
	end
	
	if gaLights then 
		for k, v in pairs ( gaLights ) do
			_gaLights[k] = v
		end
	end
	
	Call( "BeginUpdate" )

end


function OnControlValueChange ( p_sName, p_nIndex, p_gValue )

   if Call( "*:ControlExists", p_sName, p_nIndex ) then
      if p_sName == "Startup" then
         if ( Call( "GetIsEngineWithKey" ) == 1 ) then
            Call( "SendConsistMessage", ID_START, p_gValue, 0 )
            Call( "SendConsistMessage", ID_START, p_gValue, 1 )
			
			Call( "SendConsistMessage", ID_PCTCTRL, 1, 0 )
			Call( "SendConsistMessage", ID_PCTCTRL, 1, 1 )
			
         end
      end
      
      if p_sName == "Headlights" then
		checkLights( p_gValue )
      end

      
      Call( "*:SetControlValue", p_sName, p_nIndex, p_gValue );
   end
end


function Update ( p_nInterval )


	lnRPM = math.floor ( Call( "*:GetControlValue", "RPM", 0 ))
	lnRPM_change = Call( "*:GetControlValue", "RPMDelta", 0 )
		
	lnIsPlayer = Call( "GetIsEngineWithKey" )

	if lnIsPlayer == 0 then
		lnLights = Call("GetSpeed")
		
		if lnLights < -0.1 then
			lnLights = 2
		elseif lnLights > 0.1 then
			lnLights = 1
		end
	
		if lnLights ~= gnPLights then
			checkLights( lnLights )
			gnPLights = lnLights
		end
	end
	
	lnRPMpct = rpm_as_pct( lnRPM )
		
	if lnIsPlayer == 1 then
		gnThrottle = Call( "*:GetControlValue", "Regulator", 0 )
	elseif not gbUsePctRPM then	
		gnThrottle = lnRPMpct
	end

	-- RPM to smoke rate. Smoke range is from 0.1 at idle, to 0.005 at full RPM
	-- use RPM as percentage of RPM_RANGE
	-- use that as a multipler for SM_RANGE
	-- subtract the result from SM_MIN
	-- use rpm percent above idle for multiplier for smoke
	-- use throttle position as further multiplier 
	
	lnRPMRate = 1 - (( 1 - lnRPMpct )^2 )
	lnThrtWant = gnThrottle - lnRPMpct
	lnAdjRate = lnRPMRate + lnThrtWant
	
	if lnAdjRate < 0 then
		lnAdjRate = 0
	end
	if lnAdjRate > 1 then
		lnAdjRate = 1
	end
	
	gnEmitRate = SM_MIN - (SM_RANGE * lnAdjRate )
	lnColour = SC_MIN + ( SC_RANGE* lnAdjRate )
	
					
	lnSCR = lnColour * SMOKE_PCT_R
	lnSCG = lnColour * SMOKE_PCT_G
	lnSCB = lnColour * SMOKE_PCT_B
	
	for lk, lv in pairs( gaEmitters ) do
		Call ( lv..":SetEmitterColour", lnSCR, lnSCG, lnSCB );
		Call ( lv..":SetEmitterRate", gnEmitRate );
	end
	

	if lnIsPlayer == 1 then
			if ( lnRPM ~= gnPRPM ) then
				-- legacy rpm messages
				Call( "SendConsistMessage", ID_RPM, lnRPM, 1 )
				Call( "SendConsistMessage", ID_RPM, lnRPM, 0 )
				
				Call( "SendConsistMessage", ID_RPMPCT, rpm_as_pct( lnRPM ), 1 )
				Call( "SendConsistMessage", ID_RPMPCT, rpm_as_pct( lnRPM ), 0 )				
		
			end
			if ( lnRPM_change ~= gnPRPMDelta ) then
				Call( "SendConsistMessage", ID_RPMD, lnRPM_change, 1 )
				Call( "SendConsistMessage", ID_RPMD, lnRPM_change, 0 )
			end
			if ( gnThrottle ~= gnPThrottle ) then
				Call( "SendConsistMessage", ID_THROTTLEPCT, gnThrottle, 1 )
				Call( "SendConsistMessage", ID_THROTTLEPCT, gnThrottle, 0 )
			end			
						
	end
	
	gnPRPM = lnRPM
	gnPRPMDelta = lnRPM_change
	gnPThrottle = gnThrottle

end

-- Core sim function executed if a consist message is received.

function OnConsistMessage ( p_nMsg, p_gValue, p_nDir )

	if ( Call( "*:GetIsEngineWithKey" ) == 0 ) then
		if p_nMsg == ID_RPM or p_nMsg == ID_RPMPCT then
			if gbUsePctRPM then
				lnRPM = pct_as_rpm( p_gValue ) + gnRPMDrift
			else
				lnRPM = p_gValue  + gnRPMDrift
			end
			
			if lnRPM < RPM_MIN then 
				lnRPM = RPM_MIN
			end
			if lnRPM > RPM_MAX then
				lnRPM = RPM_MAX
			end
			Call( "*:SetControlValue", "RPM", 0, lnRPM )
		elseif ( p_nMsg == ID_RPMD ) then
			Call( "*:SetControlValue", "RPMDelta", 0, p_gValue )
		elseif ( p_nMsg == ID_START ) then
			Call( "*:SetControlValue", "Startup", 0, p_gValue )
		elseif ( p_nMsg == ID_PCTCTRL ) then
			gbUsePctRPM = true
		elseif ( p_nMsg == ID_THROTTLEPCT ) then
			gnThrottle = p_gValue
		end
	end

	Call( "SendConsistMessage", p_nMsg, p_gValue, p_nDir )



end
